package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.entity.ThirdCurrentWellConditionView;
import pps.core.prediction.entity.ThirdPhotovoltaicPowerView;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.mapper.ThirdCurrentWellConditionViewMapper;
import pps.core.prediction.mapper.ThirdPhotovoltaicPowerViewMapper;
import pps.core.prediction.service.data.third_active_power.GetThirdActivePowerViewOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionViewOutput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerViewOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 第三方数据接入Cloud模块
 *
 * @author ZWT
 * @date 2024/03/19 10:31
 */
@XText("第三方数据接入Cloud模块")
@XService
public class ThirdDataAccessCloudServiceImpl implements IThirdDataAccessCloudService {

    /**
     * 第三方有功功率定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdActivePowerJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA,
                new HashMap<>(0));
        List<GetThirdActivePowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdActivePowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        List<ThirdActivePowerView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdActivePowerViewOutput o : outputs) {
            dtoList.add(
                    ThirdActivePowerView.builder()
                            .stationName(o.getStationName())
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(o.getPhotovoltaicPower())
                            .meterPower(o.getMeterPower())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方当日油井状态定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdCurrentWellConditionJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA,
                new HashMap<>(0));
        List<GetThirdCurrentWellConditionViewOutput> outputs = JSON.parseArray(resultStr, GetThirdCurrentWellConditionViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        //先删后增
        List<ThirdCurrentWellConditionView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdCurrentWellConditionViewOutput o : outputs) {
            dtoList.add(
                    ThirdCurrentWellConditionView.builder()
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .wellNumber(o.getWellNumber())
                            .nextTime(o.getNextTime())
                            .wellStatus(o.getWellStatus())
                            .cumulativeProduction(o.getCumulativeProduction())
                            .runningTime(o.getRunningTime())
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdCurrentWellConditionViewMapper mapper = context.getBean(ThirdCurrentWellConditionViewMapper.class);
            mapper.deleteByParams(dtoList);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方光伏日发电量定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdPhotovoltaicPowerJob(XContext context) {
        Map<String, Object> param = new HashMap<>(4);
        param.put("Calc_Date", DateUtil.yesterday().toString("yyyy-MM-dd"));
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_STATION_DAILY_PV_DATA,
                param);
        List<GetThirdPhotovoltaicPowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdPhotovoltaicPowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        List<ThirdPhotovoltaicPowerView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdPhotovoltaicPowerViewOutput output : outputs) {
            dtoList.add(
                    ThirdPhotovoltaicPowerView.builder()
                            .stationName(output.getStationName())
                            .createDate(output.getCreateDate())
                            .photovoltaicPower(output.getPhotovoltaicPower())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdPhotovoltaicPowerViewMapper mapper = context.getBean(ThirdPhotovoltaicPowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }
}
